@@ -23,7 +23,7 @@ from gettext import gettext as _
 from pickle import dump, load
 from urllib.parse import urlparse
 from time import time
-import json
+from getpass import getuser
 from signal import signal, SIGINT, SIGTERM
 
 from eolie.settings import Settings
@@ -36,6 +36,7 @@ from eolie.database_exceptions import DatabaseExceptions
 from eolie.database_settings import DatabaseSettings
 from eolie.database_phishing import DatabasePhishing
 from eolie.sqlcursor import SqlCursor
+from eolie.context import Context
 from eolie.search import Search
 from eolie.download_manager import DownloadManager
 from eolie.menu_pages import PagesMenu
@@ -51,6 +52,8 @@ class Application(Gtk.Application):
         Eolie application
     """
 
+    __FAVICONS_PATH = "/tmp/eolie_%s" % getuser()
+
     def __init__(self, version, extension_dir):
         """
             Create application
@@ -59,6 +62,8 @@ class Application(Gtk.Application):
         """
         self.__version = version
         self.__state_cache = []
+        self.__pinned = []
+        self.__ephemeral_context = None
         signal(SIGINT, lambda a, b: self.quit())
         signal(SIGTERM, lambda a, b: self.quit())
         # Set main thread name
@@ -177,32 +182,21 @@ class Application(Gtk.Application):
                 self.__unity.set_property("progress", fraction)
                 self.__unity.set_property("progress_visible", fraction != 1.0)
 
-    def set_profiles(self):
+    def add_to_pinned(self, uri):
         """
-            Set profiles
+            Add uri to pinned pages
+            @param uri as str
         """
-        try:
-            f = Gio.File.new_for_path(EOLIE_DATA_PATH + "/profiles.json")
-            if f.query_exists():
-                (status, contents, tag) = f.load_contents(None)
-                self.__profiles = json.loads(contents.decode("utf-8"))
-            else:
-                PROFILES = {"default": _("Default"),
-                            "social": _("Social networks"),
-                            "work": _("Work"),
-                            "shopping": _("Shopping"),
-                            "personal": _("Personal"),
-                            "finance": _("Finance"),
-                            "sport": _("Sport")}
-                content = json.dumps(PROFILES)
-                f.replace_contents(content.encode("utf-8"),
-                                   None,
-                                   False,
-                                   Gio.FileCreateFlags.REPLACE_DESTINATION,
-                                   None)
-                self.__profiles = PROFILES
-        except Exception as e:
-            Logger.error("Application::set_profiles(): %s", e)
+        if uri not in self.__pinned:
+            self.__pinned.append(uri)
+
+    def remove_from_pinned(self, uri):
+        """
+            Remove uri from pinned pages
+            @param uri as str
+        """
+        if uri in self.__pinned:
+            self.__pinned.remove(uri)
 
     def quit(self, vacuum=False):
         """
@@ -235,12 +229,12 @@ class Application(Gtk.Application):
             Gio.Application.quit(self)
 
     @property
-    def profiles(self):
+    def pinned(self):
         """
-            Get profiles
-            @return {}
+            Get pinned pages
+            @return [str]
         """
-        return self.__profiles
+        return self.__pinned
 
     @property
     def start_page(self):
@@ -257,6 +251,17 @@ class Application(Gtk.Application):
             value = self.search.uri
         return value
 
+    @property
+    def ephemeral_context(self):
+        """
+            Return ephemeral context
+            @return WebKit2.WebContext
+        """
+        if self.__ephemeral_context is None:
+            self.__ephemeral_context = WebKit2.WebContext.new_ephemeral()
+            Context(self.__ephemeral_context)
+        return self.__ephemeral_context
+
     @property
     def default_style_sheet(self):
         """
@@ -289,6 +294,13 @@ class Application(Gtk.Application):
         """
         return self.__extension_dir
 
+    @property
+    def favicons_path(self):
+        """
+            Cookies sqlite DB path
+        """
+        return self.__FAVICONS_PATH
+
 #######################
 # PRIVATE             #
 #######################
@@ -365,8 +377,8 @@ class Application(Gtk.Application):
             except:
                 pass
 
-        # Init profiles
-        self.set_profiles()
+        # Init default context
+        Context(WebKit2.WebContext().get_default())
 
         shortcut_action = Gio.SimpleAction.new('shortcut',
                                                GLib.VariantType.new('s'))
@@ -425,8 +437,6 @@ class Application(Gtk.Application):
                                    ["<Control>KP_0", "<Control>0"])
         self.set_accels_for_action("win.shortcut::mse_enabled",
                                    ["<Control>m"])
-        self.set_accels_for_action("win.shortcut::add_dot_com",
-                                   ["<Control>Return"])
 
     def __vacuum(self):
         """
@@ -506,6 +516,8 @@ class Application(Gtk.Application):
                 window_states.append(window_state)
             dump(window_states,
                  open(EOLIE_DATA_PATH + "/session_states.bin", "wb"))
+            dump(self.__pinned,
+                 open(EOLIE_DATA_PATH + "/pinned.bin", "wb"))
         except Exception as e:
             Logger.error("Application::__save_state(): %s", e)
 
@@ -526,7 +538,10 @@ class Application(Gtk.Application):
         """
         size = (800, 600)
         maximized = False
+        pinned = []
         try:
+            pinned = load(
+                open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
             window_states = load(
                 open(EOLIE_DATA_PATH + "/session_states.bin", "rb"))
             if self.settings.get_value("remember-session"):
@@ -539,6 +554,8 @@ class Application(Gtk.Application):
                     i = 0 if foreground else 1
                     for (uri, title, atime,
                          ephemeral, state) in window_state["states"]:
+                        if uri in pinned:
+                            pinned.remove(uri)
                         loading_type = wanted_loading_type(i)
                         webkit_state = WebKit2.WebViewSessionState(
                             GLib.Bytes.new(state))
@@ -557,8 +574,27 @@ class Application(Gtk.Application):
                 maximized = window_states[0]["maximized"]
         except Exception as e:
             Logger.error("Application::__create_initial_windows(): %s", e)
-        if not self.get_windows():
-            self.get_new_window(size, maximized)
+        # Create a window if None available and setup pinned pages
+        # We already removed saved states from pinned
+        windows = self.get_windows()
+        if windows:
+            window = windows[0]
+        else:
+            window = self.get_new_window(size, maximized)
+            window.container.add_webview(
+                            self.start_page,
+                            LoadingType.FOREGROUND,
+                            False)
+        for uri in pinned:
+            window.container.add_webview(uri,
+                                         LoadingType.BACKGROUND,
+                                         False)
+        # Really setup pinned
+        try:
+            self.__pinned = load(
+                    open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
+        except:
+            self.__pinned = []
 
     def __on_handle_local_options(self, app, options):
         """
@@ -605,10 +641,10 @@ class Application(Gtk.Application):
                     else:
                         uri = "http://%s" % uri
                 loading_type = wanted_loading_type(i)
-                items.append((uri, uri, 0, ephemeral, None, loading_type))
+                items.append((uri, uri, None, ephemeral, None, loading_type))
                 i += 1
             active_window.container.add_webviews(items)
-            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
+            # https://gitlab.gnome.org/GNOME/gtk/issues/624
             monotonic_time = int(GLib.get_monotonic_time() / 1000)
             active_window.present_with_time(monotonic_time)
         # Add default start page
