@@ -15,8 +15,7 @@ from gi.repository import GLib, Gtk, Gio, WebKit2, Gdk
 from urllib.parse import urlparse
 from time import time
 
-from eolie.define import App, LoadingType, EOLIE_DATA_PATH
-from eolie.define import COOKIES_PATH
+from eolie.define import App, LoadingType
 from eolie.utils import get_ftp_cmd
 from eolie.logger import Logger
 
@@ -29,18 +28,13 @@ class WebViewNavigation:
 
     __MIMES = ["text/html", "text/xml", "application/xhtml+xml",
                "x-scheme-handler/http", "x-scheme-handler/https",
-               "multipart/related", "application/x-mimearchive"]
+               "multipart/related", "application/x-mimearchive",
+               "application/x-extension-html"]
 
-    def __init__(self, related_view):
+    def __init__(self):
         """
             Init navigation
-            @param related_view as WebView
         """
-        self.__related_view = related_view
-        if related_view is None:
-            self.__profile = None
-        else:
-            self.__profile = related_view.profile
         self.__insecure_content_detected = False
         self.connect("decide-policy", self.__on_decide_policy)
         self.connect("insecure-content-detected",
@@ -50,8 +44,6 @@ class WebViewNavigation:
         self.connect("notify::title", self.__on_title_changed)
         self.connect("notify::uri", self.__on_uri_changed)
 
-        self.__switch_profile(None)
-
     def load_uri(self, uri):
         """
             Load uri
@@ -93,56 +85,8 @@ class WebViewNavigation:
                 self.reset_bad_tls()
                 self.__insecure_content_detected = False
             self.stop_loading()
-            self.update_settings_for_uri(uri)
             GLib.idle_add(WebKit2.WebView.load_uri, self, uri)
 
-    def update_settings_for_uri(self, uri):
-        """
-            Update internal settings for URI
-            @param uri as str
-        """
-        parsed = urlparse(uri)
-        http_scheme = parsed.scheme in ["http", "https"]
-        App().history.set_page_state(uri)
-        # self.__switch_profile(uri)
-        self.__update_bookmark_metadata(uri)
-        self.content_manager.remove_all_style_sheets()
-        # Can't find a way to block content for ephemeral views
-        if App().settings.get_value("adblock") and\
-                not App().adblock_exceptions.find_parsed(parsed) and\
-                http_scheme and\
-                self.content_manager is not None:
-            self.content_manager.add_style_sheet(
-                App().default_style_sheet)
-            rules = App().adblock.get_css_rules(uri)
-            user_style_sheet = WebKit2.UserStyleSheet(
-                rules,
-                WebKit2.UserContentInjectedFrames.ALL_FRAMES,
-                WebKit2.UserStyleLevel.USER,
-                None,
-                None)
-            self.content_manager.add_style_sheet(user_style_sheet)
-        user_agent = App().websettings.get_user_agent(uri)
-        settings = self.get_settings()
-        if user_agent:
-            settings.set_user_agent(user_agent)
-        else:
-            settings.set_user_agent_with_application_details("Eolie",
-                                                             None)
-        # Setup image blocker
-        block_image = http_scheme and\
-            App().settings.get_value("imageblock") and\
-            not App().image_exceptions.find_parsed(parsed)
-        self.set_setting("auto-load-images", not block_image)
-
-    @property
-    def profile(self):
-        """
-            Get profile
-            @return str
-        """
-        return self.__profile
-
 #######################
 # PROTECTED           #
 #######################
@@ -155,10 +99,15 @@ class WebViewNavigation:
         parsed = urlparse(self.uri)
         if event == WebKit2.LoadEvent.STARTED:
             self.emit("uri-changed", self.uri)
+            if parsed.scheme in ["http", "https"]:
+                self._initial_uri = self.uri.rstrip('/')
+            else:
+                self._initial_uri = None
+            self.__update_bookmark_metadata(self.uri)
         elif event == WebKit2.LoadEvent.REDIRECTED:
-            # Block ads
+            self.__update_bookmark_metadata(self.uri)
+            # Block ads, useful for some site
             if App().settings.get_value("adblock") and\
-                    webview.__related_view is not None and\
                     parsed.scheme in ["http", "https"] and\
                     not App().adblock_exceptions.find_parsed(parsed):
                 if App().adblock.is_netloc_blocked(parsed.netloc) or\
@@ -171,6 +120,11 @@ class WebViewNavigation:
                     return
         elif event == WebKit2.LoadEvent.COMMITTED:
             self.emit("uri-changed", self.uri)
+            App().history.set_page_state(self.uri)
+            if self._initial_uri != self.uri:
+                self.__update_bookmark_metadata(self.uri)
+            self.__set_imgblock(self.uri)
+            self.__set_adblock(self.uri)
             self.update_zoom_level()
         elif event == WebKit2.LoadEvent.FINISHED:
             self.update_spell_checking(self.uri)
@@ -216,22 +170,55 @@ class WebViewNavigation:
             App().bookmarks.set_access_time(uri, round(time(), 2))
             App().bookmarks.set_more_popular(uri)
 
-    def __switch_profile(self, uri):
+    def __set_adblock(self, uri):
         """
-            Handle cookies manager
+            Set adblocker
             @param uri as str
         """
-        if self.ephemeral or self.__related_view is not None:
-            return
-        # profile = App().websettings.get_profile(uri)
-        profile = "default"
-        if self.__profile != profile:
-            self.__profile = profile
-            cookie_manager = self.get_context().get_cookie_manager()
-            path = COOKIES_PATH % (EOLIE_DATA_PATH, profile)
-            cookie_manager.set_persistent_storage(
-                path,
-                WebKit2.CookiePersistentStorage.SQLITE)
+        parsed = urlparse(uri)
+        http_scheme = parsed.scheme in ["http", "https"]
+        self.content_manager.remove_all_style_sheets()
+        # Can't find a way to block content for ephemeral views
+        if App().settings.get_value("adblock") and\
+                not App().adblock_exceptions.find_parsed(parsed) and\
+                http_scheme and\
+                self.content_manager is not None:
+            self.content_manager.add_style_sheet(
+                App().default_style_sheet)
+            rules = App().adblock.get_css_rules(uri)
+            user_style_sheet = WebKit2.UserStyleSheet(
+                rules,
+                WebKit2.UserContentInjectedFrames.ALL_FRAMES,
+                WebKit2.UserStyleLevel.USER,
+                None,
+                None)
+            self.content_manager.add_style_sheet(user_style_sheet)
+
+    def __set_user_agent(self, uri):
+        """
+            Set user agent for uri
+            @param uri as str
+        """
+        user_agent = App().websettings.get_user_agent(uri)
+        settings = self.get_settings()
+        if user_agent:
+            settings.set_user_agent(user_agent)
+        else:
+            settings.set_user_agent_with_application_details("Eolie",
+                                                             None)
+
+    def __set_imgblock(self, uri):
+        """
+            Set image blocker for uri
+            @param uri as str
+        """
+        parsed = urlparse(uri)
+        http_scheme = parsed.scheme in ["http", "https"]
+        # Setup image blocker
+        block_image = http_scheme and\
+            App().settings.get_value("imageblock") and\
+            not App().image_exceptions.find_parsed(parsed)
+        self.set_setting("auto-load-images", not block_image)
 
     def __on_run_as_modal(self, webview):
         Logger.info("WebView::__on_run_as_modal(): TODO")
@@ -268,6 +255,7 @@ class WebViewNavigation:
         """
         # Js update
         if not self.is_loading():
+            self._initial_uri = None
             uri = webview.get_property(param.name)
             # JS bookmark (Bookmarklet)
             if not uri.startswith("javascript:") and not self.error:
@@ -344,28 +332,23 @@ class WebViewNavigation:
             if navigation_uri == "about:blank":
                 self.reset_last_click_event()
                 decision.use()
-                return True
+                return False
             elif decision_type == WebKit2.PolicyDecisionType.NEW_WINDOW_ACTION:
-                self.new_page(navigation_uri, LoadingType.FOREGROUND)
-                decision.ignore()
-                return True
+                decision.use()
+                return False
             elif App().phishing.is_phishing(navigation_uri):
                 self._show_phishing_error(navigation_uri)
                 decision.ignore()
                 return True
             else:
                 self.discard_error()
+                self.__set_user_agent(navigation_uri)
                 decision.use()
                 return False
         elif mouse_button == 1:
             if decision_type == WebKit2.PolicyDecisionType.NEW_WINDOW_ACTION:
-                if self._window.modifiers == Gdk.KEY_Shift_L:
-                    loading_type = LoadingType.POPOVER
-                else:
-                    loading_type = LoadingType.FOREGROUND
-                self.new_page(navigation_uri, loading_type)
-                decision.ignore()
-                return True
+                decision.use()
+                return False
             elif self._window.modifiers == Gdk.KEY_Control_L:
                 self.new_page(navigation_uri, LoadingType.BACKGROUND)
                 decision.ignore()
@@ -379,7 +362,7 @@ class WebViewNavigation:
                 decision.ignore()
                 return True
             else:
-                self.update_settings_for_uri(navigation_uri)
+                self.__set_user_agent(navigation_uri)
                 self.discard_error()
                 decision.use()
                 return False
